%% ECCB 2014: A multimodal approach to protein remote homology detection
%
% In this demo, we show the proposed multimodal framework applied to remote
% homology detection of the SCOP 1.53 family 3.42.1.1
%
% Pietro Lovato, January 2014

clear all
close all
clc

addpath(genpath('./util/'));

% Check if libsvm, ccg and prtools packages are installed
type = exist('svmpredict', 'file');
if type ~=3
    fprintf('ERROR: could''t find libsvm in the current path\n');
    return
end
if ~exist('ccg.m', 'file')
    fprintf('ERROR: could''t find CCG code in the current path\n');
    return
end
if ~exist('prdataset.m', 'file')
    fprintf('ERROR: could''t find prtools in the current path\n');
    return
end

%% Set up parameters

% Componential counting grid size
E = [20 20];
W = [2 2];
% Equivalent number of topics for LDA
topics = 100;

% SVM parameters
sigma = 1.5.^(-2);
C = 10.^(-3);

%% Load dataset

% Load dataset and labels
load('./Dataset/seq_counts.mat')
load('./Dataset/struct_counts.mat')
load('./Dataset/labels.mat')

%% Single-modal CCG: train only on sequences

% Learn on training sequences (this can take some minutes)
fprintf('Learning ccg on training sequences...');
tstart = tic;
[pi, alpha, theta] = ccgStruct_wrapper(counts_seqtr, E, W);
telapsed = toc(tstart);
fprintf([' DONE! (' sprintf('%.1f', telapsed) 's)\n']);

% Inference on testing sequences
fprintf('Inference on test sequences...');
tstart = tic;
[~, ~, theta_te] = ccgSeqInference_wrapper(counts_seqte, ...
    E, W, pi, alpha, theta);
telapsed = toc(tstart);
fprintf([' DONE! (' sprintf('%.1f', telapsed) 's)\n']);

% Get feature vectors for classification
% This step compute the smoothed theta probabilities, as in (Perina, 2013)
clear exp_theta_tr exp_theta_te
U = padarray( ones(W),E-W,0,'post') ./ prod(W); % U^W in the paper
for d=1:size(theta,2)
    tmp = real( ifft2( fft2( reshape( theta(:,d),E )).*fft2( U ) ) );
    tmp(tmp<0)=0;
    exp_theta_tr(:,d) = tmp(:);
end
for d=1:size(theta_te,2)
    tmp = real( ifft2( fft2( reshape( theta_te(:,d),E )).*fft2( U ) ) );
    tmp(tmp<0)=0;
    exp_theta_te(:,d) = tmp(:);
end

% Classification (requires prtools)
tr = prdataset(exp_theta_tr', labels_train);
te = prdataset(exp_theta_te', labels_test);

kernel = proxm([], 'r', sigma);

w = libsvc(tr, kernel, C);
rocccg_seqOnly = 1-testc(te, w, 'auc');


%% X-CCG

% idx_corr contains the indices of training sequcences for which we know
% the corresponding structure
idxseq = sort(setdiff(1:size(counts_seqtr,2),idx_corr), 'ascend')';

str = counts_structtr;
sstr = counts_seqtr(:, idx_corr);
sste = counts_seqtr(:, idxseq);

% Learn on structures
fprintf('Learning ccg on structures...');
tstart = tic;
[pi_corr, alpha_corr, theta_corr] = ...
    ccgStruct_wrapper(str, E, W);
telapsed = toc(tstart);
fprintf([' DONE! (' sprintf('%.1f', telapsed) 's)\n']);

% Multimodal projection
fprintf('Multimodal projection...');
tstart = tic;
[xpi, xalpha, xtheta] = xccg_wrapper_partInf(sstr, sste, ...
    E, W, theta_corr, idx_corr, idxseq);
telapsed = toc(tstart);
fprintf(' DONE! (%.1fs)\n', telapsed);

% Inference
fprintf('Inference on test sequences...');
tstart = tic;
[pi_te, alpha_te, theta_te] = ccgSeqInference_wrapper(counts_seqte, ...
    E, W, xpi, xalpha, xtheta);
telapsed = toc(tstart);
fprintf(' DONE! (%.1fs)\n', telapsed);

% Smoothing
clear exp_theta_tr exp_theta_te
U = padarray( ones(W),E-W,0,'post') ./ prod(W); % U^W in the paper.
for d=1:size(xtheta,2)
    tmp = real( ifft2( fft2( reshape( xtheta(:,d),E )).*fft2( U ) ) );
    tmp(tmp<0)=0;
    exp_theta_tr(:,d) = tmp(:);
end
for d=1:size(theta_te,2)
    tmp = real( ifft2( fft2( reshape( theta_te(:,d),E )).*fft2( U ) ) );
    tmp(tmp<0)=0;
    exp_theta_te(:,d) = tmp(:);
end

% Classification
tr = prdataset(exp_theta_tr', labels_train);
te = prdataset(exp_theta_te', labels_test);

kernel = proxm([], 'r', sigma);
w = libsvc(tr, kernel, C);
rocccg = 1-testc(te, w, 'auc');

%% Single-modal LDA: train only on sequences

% Learn on sequences (this can take some minutes)
fprintf('Learning lda on structures...');
tstart = tic;
[wt, td] = ...
    ldaStruct_wrapper(counts_seqtr, topics);
telapsed = toc(tstart);
fprintf(' DONE! (%.1fs)\n', telapsed);

% Inference
fprintf('Inference on test sequences...');
tstart = tic;
[wt_te, td_te] = ldaSeqInference_wrapper(counts_seqte, ...
    topics, wt, td);
telapsed = toc(tstart);
fprintf(' DONE! (%.1fs)\n', telapsed);

% Classification
tr = prdataset(td', labels_train);
te = prdataset(td_te', labels_test);

kernel = proxm([], 'r', sigma);
w = libsvc(tr, kernel, C);
roclda_seqOnly = 1-testc(te, w, 'auc');


%% Multimodal LDA
% Learn on structures (this can take some minutes)
fprintf('Learning lda on structures...');
tstart = tic;
[wt_corr, td_corr] = ...
    ldaStruct_wrapper(str, topics);
telapsed = toc(tstart);
fprintf(' DONE! (%.1fs)\n', telapsed);

% Transfer
fprintf('Model transfer...');
tstart = tic;
[xwt, xtd] = xlda_wrapper_partInf(sstr, sste, ...
    topics, td_corr, idx_corr, idxseq);
telapsed = toc(tstart);
fprintf(' DONE! (%.1fs)\n', telapsed);

% Inference
fprintf('Inference on test sequences...');
tstart = tic;
[wt_te, td_te] = ldaSeqInference_wrapper(counts_seqte, ...
    topics, xwt, xtd);
telapsed = toc(tstart);
fprintf(' DONE! (%.1fs)\n', telapsed);

% Classification
tr = prdataset(xtd', labels_train);
te = prdataset(td_te', labels_test);

kernel = proxm([], 'r', sigma);
w = libsvc(tr, kernel, C);
roclda = 1-testc(te, w, 'auc');

%% Display results

fprintf('------------------------------------------------------\n');
fprintf('Sequences modality only: CCG roc score = %.3f\n', ...
    rocccg_seqOnly);

fprintf('Multimodality:           CCG roc score = %.3f\n', ...
    rocccg);

fprintf('\n');
fprintf('Sequences modality only: LDA roc score = %.3f\n', ...
    roclda_seqOnly);

fprintf('Multimodality:           LDA roc score = %.3f\n', ...
    roclda);




